﻿using UnityEngine;
using UnityEditor;
using UnityEngine.TestTools;
using NUnit.Framework;
using System.Collections;
using Windows.Kinect;
using System.Runtime.InteropServices;

public class SourceBufferTests
{
    const int DepthWidth = 512,
              DepthHeight = 424,
              ColorWidth = 1920,
              ColorHeight = 1080;

    [Test]
	public void SourceBufferTestsSimplePasses()
    {
        // Use the Assert class to test conditions.
        
        var sensor = KinectSensor.GetDefault();
        Assert.IsNotNull(sensor, "No ready kinect found.");
        sensor.Open();
        Assert.IsTrue(sensor.IsOpen, "Kinect sensor has problem.");

        var coordinateMapper = sensor.CoordinateMapper;
        using (var sourceFrameReader = sensor.OpenMultiSourceFrameReader(
            FrameSourceTypes.Color |
            FrameSourceTypes.Depth |
            FrameSourceTypes.BodyIndex |
            FrameSourceTypes.Body))
        {
            while (sourceFrameReader == null) ;

            var sourceFrame = sourceFrameReader.AcquireLatestFrame();

            SourceBuffer.UpdateBuffers(sourceFrame);

            ColorBufferTest(sourceFrame);
        }

        sensor.Close();
    }

    private void ColorBufferTest(MultiSourceFrame sourceFrame)
    {
        var colorBuffer = new byte[ColorWidth * ColorHeight * 4];
        using (var colorFrame = sourceFrame.ColorFrameReference.AcquireFrame())
            if (colorFrame != null)
            {
                var ColorData = GCHandle.Alloc(colorBuffer, GCHandleType.Pinned);
                colorFrame.CopyConvertedFrameDataToIntPtr(ColorData.AddrOfPinnedObject(), (uint)colorBuffer.Length, ColorImageFormat.Rgba);
                ColorData.Free();
            }

        Assert.AreEqual(colorBuffer, SourceBuffer.ColorBuffer, "ColorBufferTest failed.");
    }

    // A UnityTest behaves like a coroutine in PlayMode
    // and allows you to yield null to skip a frame in EditMode
    [UnityTest]
	public IEnumerator SourceBufferTestsWithEnumeratorPasses()
    {
		// Use the Assert class to test conditions.
		// yield to skip a frame
		yield return null;
	}
}
